using System;
using Server;
using Server.Targeting;
using Server.Items;

namespace Server.Spells.Druid
{
   public class BlendWithForestSpell : DruidicSpell
   {
      private static SpellInfo m_Info = new SpellInfo(
            "Blend With Forest", "Kes Ohm",
            SpellCircle.Sixth,
            206,
            9002,
            false,
            Reagent.Bloodmoss,
            Reagent.Nightshade
         );

      public override double CastDelay{ get{ return 2.0; } }
      public override double RequiredSkill{ get{ return 75.0; } }
      public override int RequiredMana{ get{ return 60; } }

      public BlendWithForestSpell( Mobile caster, Item scroll ) : base( caster, scroll, m_Info )
      {
      }

      public override void OnCast()
      {
         Caster.Target = new InternalTarget( this );
      }

      public void Target( Mobile m )
      {
         if ( !Caster.CanSee( m ) )
         {
            Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
         }
         else if ( CheckBSequence( m ) )
         {
            SpellHelper.Turn( Caster, m );

            Effects.SendLocationParticles( EffectItem.Create( new Point3D( m.X, m.Y, m.Z + 16 ), Caster.Map, EffectItem.DefaultDuration ), 0x376A, 10, 15, 5045 );

            m.Hidden = true;

            TimeSpan duration = TimeSpan.FromSeconds( Caster.Skills[CastSkill].Value * 1.5 );

            new InternalTimer( m, duration ).Start();
         }

         FinishSequence();
      }

      private class InternalTimer : Timer
      {
         private Mobile m_Mobile;

         public InternalTimer( Mobile m, TimeSpan duration ) : base( duration )
         {
            m_Mobile = m;
         }

         protected override void OnTick()
         {
            m_Mobile.RevealingAction();
         }
      }

      public class InternalTarget : Target
      {
         private BlendWithForestSpell m_Owner;

         public InternalTarget( BlendWithForestSpell owner ) : base( 12, false, TargetFlags.Beneficial )
         {
            m_Owner = owner;
         }

         protected override void OnTarget( Mobile from, object o )
         {
            if ( o is Mobile )
            {
               m_Owner.Target( (Mobile)o );
            }
         }

         protected override void OnTargetFinish( Mobile from )
         {
            m_Owner.FinishSequence();
         }
      }
   }
}
